﻿#include <iostream>
#include <vector>
#include <map>
#include <queue>

using namespace std;

const int INF = 999999;

// 定义路由节点
struct Router {
    int id;
    map<int, int> links; // 链路信息，key为相邻节点id，value为链路成本
};

// 定义网络拓扑
vector<Router> routers;

// 计算最短路径
map<int, int> dijkstra(int start) {
    map<int, int> distance; // 起始节点到各节点的最短距离
    priority_queue<pair<int, int>, vector<pair<int, int>>, greater<pair<int, int>>> pq; // 最小堆，pair<距离, 节点>

    // 初始化距离，将起始节点距离设为0，其余节点设为INF
    for (const auto& router : routers) {
        distance[router.id] = INF;
    }
    distance[start] = 0;

    pq.push({ 0, start });

    while (!pq.empty()) {
        int current = pq.top().second;
        int current_distance = pq.top().first;
        pq.pop();

        if (current_distance > distance[current]) {
            continue;
        }

        // 更新相邻节点的距离
        for (const auto& neighbor : routers[current - 1].links) { // 注意索引从0开始,neighbor是每一个router的map
            int new_distance = current_distance + neighbor.second;

            if (new_distance < distance[neighbor.first]) {//距离更短则替换
                distance[neighbor.first] = new_distance;
                pq.push({ new_distance, neighbor.first });
            }
        }
    }

    return distance;
}

// 构造路由表
void buildRoutingTable(int start) {
    map<int, int> shortestDistances = dijkstra(start);

    cout << "路由器 " << start << " 路由表:" << endl;
    cout << "目标\t\t成本" << endl;
    for (const auto& entry : shortestDistances) {
        if (entry.first != start && entry.second != INF) {
            // 如果最短距离是 INF，可能没有直接的链接
            cout << entry.first << "\t\t" << entry.second << endl;
        }
    }
}

int main() {
    int num_nodes;
    cout << "输入网络中的节点数量: ";
    cin >> num_nodes;

    // 手动输入路由节点信息
    for (int i = 1; i <= num_nodes; ++i) {
        int id, num_links;
        cout << "输入路由器 " << i << " 的ID: ";
        cin >> id;

        cout << "输入路由器 " << i << " 的邻居数量: ";
        cin >> num_links;

        Router router;
        router.id = id;

        cout << "输入路由器 " << i << " 的邻居信息 (邻居ID和链路成本):" << endl;
        for (int j = 0; j < num_links; ++j) {
            int neighbor, cost;
            cin >> neighbor >> cost;
            router.links[neighbor] = cost;
        }

        routers.push_back(router);
    }

    // 构造网络拓扑并计算最短路径
    for (int i = 1; i <= num_nodes; i++)
        buildRoutingTable(i);

    return 0;
}
